==> muse_0.5.2.orig.tar.gz: muse-0.5.2/tempo.cpp <==
//=========================================================
//  MusE
//  Linux Music Editor
//  $Id: tempo.cpp,v 1.1 2002/01/30 14:10:07 muse Exp $
//
//  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
//=========================================================

#include <stdio.h>
#include <errno.h>

#include "tempo.h"
#include "globals.h"
#include "xml.h"

TempoList tempomap;

//---------------------------------------------------------
//   TempoList
//---------------------------------------------------------

TempoList::TempoList()
      {
      _tempo   = 500000;
      insert(std::pair<const int, TEvent*> (MAX_TICK, new TEvent(_tempo, 0)));
      _tempoSN = 1;
      useList  = true;
      }

//---------------------------------------------------------
//   add
//---------------------------------------------------------

void TempoList::add(int tick, int tempo)
      {
// printf("TempoList::add(%d %d)\n", tick, tempo);
      iTEvent e = upper_bound(tick);
      assert(e != end());     // happens if tick > MAX_TICK

      if (tick == e->second->tick)
            e->second->tempo = tempo;
      else {
            TEvent* ne = e->second;
            TEvent* ev = new TEvent(ne->tempo, ne->tick);
            ne->tempo = tempo;
            ne->tick  = tick;
            insert(std::pair<const int, TEvent*> (tick, ev));
            }
      normalize();
      }

//---------------------------------------------------------
//   TempoList::normalize
//---------------------------------------------------------

void TempoList::normalize()
      {
      double time = 0;
      for (iTEvent e = begin(); e != end(); ++e) {
            e->second->time = time;
            int delta = e->first - e->second->tick;
            time += double(delta) / (division * (1000000.0/e->second->tempo));
            }
      }

//---------------------------------------------------------
//   TempoList::dump
//---------------------------------------------------------

void TempoList::dump() const
      {
      printf("\nTempoList:\n");
      for (ciTEvent i = begin(); i != end(); ++i) {
            printf("%6d %06d Tempo %6d %f\n",
               i->first, i->second->tick, i->second->tempo,
               i->second->time);
            }
      }

//---------------------------------------------------------
//   clear
//---------------------------------------------------------

void TempoList::clear()
      {
      for (iTEvent i = begin(); i != end(); ++i)
            delete i->second;
      std::map<int, TEvent*, std::less<int> >::clear();
      insert(std::pair<const int, TEvent*> (MAX_TICK, new TEvent(500000, 0)));
      ++_tempoSN;
      }

//---------------------------------------------------------
//   tempo
//---------------------------------------------------------

int TempoList::tempo(int tick) const
      {
      if (useList) {
            ciTEvent i = upper_bound(tick);
            if (i == end()) {
                  printf("no TEMPO\n");
                  return 1000;
                  }
            return i->second->tempo;
            }
      else
            return _tempo;
      }

//---------------------------------------------------------
//   del
//---------------------------------------------------------

void TempoList::del(int tick)
      {
// printf("TempoList::del(%d)\n", tick);
      iTEvent e = find(tick);
      if (e == end()) {
            printf("TempoList::del(%d): not found\n", tick);
            return;
            }
      del(e);
      ++_tempoSN;
      }

void TempoList::del(iTEvent e)
      {
      iTEvent ne = e;
      ++ne;
      if (ne == end()) {
            printf("TempoList::del() HALLO\n");
            return;
            }
      ne->second->tempo = e->second->tempo;
      ne->second->tick  = e->second->tick;
      erase(e);
      normalize();
      ++_tempoSN;
      }

//---------------------------------------------------------
//   change
//---------------------------------------------------------

void TempoList::change(int tick, int newTempo)
      {
      iTEvent e = find(tick);
      assert(e != end());
      e->second->tempo = newTempo;
      normalize();
      ++_tempoSN;
      }

//---------------------------------------------------------
//   setTempo
//    called from transport window
//    & slave mode tempo changes
//---------------------------------------------------------

void TempoList::setTempo(int tick, int newTempo)
      {
// printf("set tempo useList:%d %d, diff %3d\n", useList, newTempo, newTempo-_tempo);
      if (useList)
            add(tick, newTempo);
      else
            _tempo = newTempo;
      ++_tempoSN;
      }

//---------------------------------------------------------
//   addTempo
//---------------------------------------------------------

void TempoList::addTempo(int t, int tempo)
      {
      add(t, tempo);
      ++_tempoSN;
      }

//---------------------------------------------------------
//   delTempo
//---------------------------------------------------------

void TempoList::delTempo(int tick)
      {
      del(tick);
      ++_tempoSN;
      }

//---------------------------------------------------------
//   changeTempo
//---------------------------------------------------------

void TempoList::changeTempo(int tick, int newTempo)
      {
      change(tick, newTempo);
      ++_tempoSN;
      }

//---------------------------------------------------------
//   setMasterFlag
//---------------------------------------------------------

bool TempoList::setMasterFlag(int /*tick*/, bool val)
      {
      if (useList != val) {
            useList = val;
            ++_tempoSN;
            return true;
            }
      return false;
      }

//---------------------------------------------------------
//   tick2time
//---------------------------------------------------------

double TempoList::tick2time(int tick, int* sn = 0) const
      {
      double t;
      if (useList) {
            ciTEvent i = upper_bound(tick);
            int delta = tick - i->second->tick;
            t = i->second->time + double(delta) / (division * (1000000.0/i->second->tempo));
            }
      else {
            t = (double(tick) * double(_tempo)) / (double(division) * 1000000.0);
            }
      if (sn)
            *sn = _tempoSN;
      return t;
      }

double TempoList::tick2timeLC(int tick, int* sn) const
      {
      if (*sn == _tempoSN) {
            return 0.0;
            }
      double t;
      if (useList) {
            ciTEvent i = upper_bound(tick);
            int delta = tick - i->second->tick;
            t = i->second->time + double(delta) / (division * (1000000.0/i->second->tempo));
            }
      else {
            t = (double(tick) * double(_tempo)) / (double(division) * 1000000.0);
            }
      *sn = _tempoSN;
      return t;
      }

//---------------------------------------------------------
//   tick2time
//---------------------------------------------------------

double TempoList::tick2time(int tick, double time, int* sn) const
      {
      double t;
      if (*sn == _tempoSN)
            t = time;
      else {
            if (useList) {
                  ciTEvent i = upper_bound(tick);
                  int delta = tick - i->second->tick;
                  t = i->second->time + double(delta) / (division * (1000000.0/i->second->tempo));
                  }
            else {
                  t = (double(tick) * double(_tempo)) / (double(division) * 1000000.0);
                  }
            *sn = _tempoSN;
            }
      return t;
      }

//---------------------------------------------------------
//   time2tick
//---------------------------------------------------------

int TempoList::time2tick(double time, int* sn = 0) const
      {
      int tick;
      if (useList) {
            ciTEvent e;
            for (e = begin(); e != end();) {
                  ciTEvent ee = e;
                  ++ee;
                  if (ee == end())
                        break;
                  if (time < ee->second->time)
                        break;
                  e = ee;
                  }
            int te = e->second->tempo;
            double dtime = time - e->second->time;
            tick = e->second->tick + int(dtime * division * (1000000.0/te));
            }
      else
            tick = int((time * division * 1000000.0 / double(_tempo)) + 0.5);
      if (sn)
            *sn = _tempoSN;
      return tick;
      }

//---------------------------------------------------------
//   time2tick
//---------------------------------------------------------

int TempoList::time2tick(double time, int t, int* sn = 0) const
      {
      int tick;
      if (*sn == _tempoSN) {
            tick = t;
            }
      else {
            if (useList) {
                  ciTEvent e;
                  for (e = begin(); e != end();) {
                        ciTEvent ee = e;
                        ++ee;
                        if (ee == end())
                              break;
                        if (time < ee->second->time)
                              break;
                        e = ee;
                        }
                  int te = e->second->tempo;
                  double dtime = time - e->second->time;
                  tick = e->second->tick + int(dtime * division * (1000000.0/te));
                  }
            else
                  tick = int((time * division * 1000000.0 / double(_tempo)) + 0.5);
            *sn = _tempoSN;
            }
      return tick;
      }

//---------------------------------------------------------
//   TempoList::write
//---------------------------------------------------------

void TempoList::write(int level, Xml& xml) const
      {
      xml.put(level++, "<tempolist fix=\"%d\">", _tempo);
      for (ciTEvent i = begin(); i != end(); ++i)
            i->second->write(level, xml, i->first);
      xml.tag(level, "/tempolist");
      }

//---------------------------------------------------------
//   TempoList::read
//---------------------------------------------------------

void TempoList::read(Xml& xml)
      {
      for (;;) {
            Xml::Token token = xml.parse();
            const QString& tag = xml.s1();
            switch (token) {
                  case Xml::Error:
                  case Xml::End:
                        return;
                  case Xml::TagStart:
                        if (tag == "tempo") {
                              TEvent* t = new TEvent();
                              int tick = t->read(xml);
                              iTEvent pos = find(tick);
                              if (pos != end())
                                    erase(pos);
                              insert(std::pair<const int, TEvent*> (tick, t));
                              }
                        else
                              xml.unknown("TempoList");
                        break;
                  case Xml::Attribut:
                        if (tag == "fix")
                              _tempo = xml.s2().toInt();
                        break;
                  case Xml::TagEnd:
                        if (tag == "tempolist") {
                              normalize();
                              ++_tempoSN;
                              return;
                              }
                  default:
                        break;
                  }
            }
      }

//---------------------------------------------------------
//   tick2samples
//---------------------------------------------------------

int TempoList::tick2samples(int tick)
      {
      return int(tick2time(tick) * sampleRate);
      }

//---------------------------------------------------------
//   samples2tick
//---------------------------------------------------------

int TempoList::samples2tick(int samples)
      {
      return time2tick(double(samples) / double(sampleRate));
      }

//---------------------------------------------------------
//   TEvent::write
//---------------------------------------------------------

void TEvent::write(int level, Xml& xml, int at) const
      {
      xml.tag(level++, "tempo at=\"%d\"", at);
      xml.intTag(level, "tick", tick);
      xml.intTag(level, "val", tempo);
      xml.tag(level, "/tempo");
      }

//---------------------------------------------------------
//   TEvent::read
//---------------------------------------------------------

int TEvent::read(Xml& xml)
      {
      int at = 0;
      for (;;) {
            Xml::Token token = xml.parse();
            const QString& tag = xml.s1();
            switch (token) {
                  case Xml::Error:
                  case Xml::End:
                        return 0;
                  case Xml::TagStart:
                        if (tag == "tick")
                              tick = xml.parseInt();
                        else if (tag == "val")
                              tempo = xml.parseInt();
                        else
                              xml.unknown("TEvent");
                        break;
                  case Xml::Attribut:
                        if (tag == "at")
                              at = xml.s2().toInt();
                        break;
                  case Xml::TagEnd:
                        if (tag == "tempo") {
                              return at;
                              }
                  default:
                        break;
                  }
            }
      return 0;
      }

==> muse_0.6.3.orig.tar.gz: muse-0.6.3/tempo.cpp <==
//=========================================================
//  MusE
//  Linux Music Editor
//  $Id: tempo.cpp,v 1.1.1.1 2003/10/29 10:05:18 wschweer Exp $
//
//  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
//=========================================================

#include <stdio.h>
#include <errno.h>
#include <cmath>

#include "tempo.h"
#include "globals.h"
#include "xml.h"

TempoList tempomap;

//---------------------------------------------------------
//   TempoList
//---------------------------------------------------------

TempoList::TempoList()
      {
      _tempo   = 500000;
      insert(std::pair<const int, TEvent*> (MAX_TICK, new TEvent(_tempo, 0)));
      _tempoSN = 1;
      _globalTempo = 100;
      useList  = true;
      }

//---------------------------------------------------------
//   add
//---------------------------------------------------------

void TempoList::add(int tick, int tempo)
      {
// printf("TempoList::add(%d %d)\n", tick, tempo);
      iTEvent e = upper_bound(tick);
//      assert(e != end());     // happens if tick > MAX_TICK

      if (tick == e->second->tick)
            e->second->tempo = tempo;
      else {
            TEvent* ne = e->second;
            TEvent* ev = new TEvent(ne->tempo, ne->tick);
            ne->tempo = tempo;
            ne->tick  = tick;
            insert(std::pair<const int, TEvent*> (tick, ev));
            }
      normalize();
      }

//---------------------------------------------------------
//   TempoList::normalize
//---------------------------------------------------------

void TempoList::normalize()
      {
      double time = 0;
      for (iTEvent e = begin(); e != end(); ++e) {
            e->second->time = time;
            int delta = e->first - e->second->tick;
            time += double(delta) / (division * _globalTempo * 10000.0/e->second->tempo);
            }
      }

//---------------------------------------------------------
//   TempoList::dump
//---------------------------------------------------------

void TempoList::dump() const
      {
      printf("\nTempoList:\n");
      for (ciTEvent i = begin(); i != end(); ++i) {
            printf("%6d %06d Tempo %6d %f\n",
               i->first, i->second->tick, i->second->tempo,
               i->second->time);
            }
      }

//---------------------------------------------------------
//   clear
//---------------------------------------------------------

void TempoList::clear()
      {
      for (iTEvent i = begin(); i != end(); ++i)
            delete i->second;
      std::map<int, TEvent*, std::less<int> >::clear();
      insert(std::pair<const int, TEvent*> (MAX_TICK, new TEvent(500000, 0)));
      ++_tempoSN;
      }

//---------------------------------------------------------
//   tempo
//---------------------------------------------------------

int TempoList::tempo(int tick) const
      {
      if (useList) {
            ciTEvent i = upper_bound(tick);
            if (i == end()) {
                  printf("no TEMPO\n");
                  return 1000;
                  }
            return i->second->tempo;
            }
      else
            return _tempo;
      }

//---------------------------------------------------------
//   del
//---------------------------------------------------------

void TempoList::del(int tick)
      {
// printf("TempoList::del(%d)\n", tick);
      iTEvent e = find(tick);
      if (e == end()) {
            printf("TempoList::del(%d): not found\n", tick);
            return;
            }
      del(e);
      ++_tempoSN;
      }

void TempoList::del(iTEvent e)
      {
      iTEvent ne = e;
      ++ne;
      if (ne == end()) {
            printf("TempoList::del() HALLO\n");
            return;
            }
      ne->second->tempo = e->second->tempo;
      ne->second->tick  = e->second->tick;
      erase(e);
      normalize();
      ++_tempoSN;
      }

//---------------------------------------------------------
//   change
//---------------------------------------------------------

void TempoList::change(int tick, int newTempo)
      {
      iTEvent e = find(tick);
//      assert(e != end());
      e->second->tempo = newTempo;
      normalize();
      ++_tempoSN;
      }

//---------------------------------------------------------
//   setTempo
//    called from transport window
//    & slave mode tempo changes
//---------------------------------------------------------

void TempoList::setTempo(int tick, int newTempo)
      {
// printf("set tempo useList:%d %d, diff %3d\n", useList, newTempo, newTempo-_tempo);
      if (useList)
            add(tick, newTempo);
      else
            _tempo = newTempo;
      ++_tempoSN;
      }

//---------------------------------------------------------
//   setGlobalTempo
//---------------------------------------------------------

void TempoList::setGlobalTempo(int val)
      {
      _globalTempo = val;
      ++_tempoSN;
      normalize();
      }

//---------------------------------------------------------
//   addTempo
//---------------------------------------------------------

void TempoList::addTempo(int t, int tempo)
      {
      add(t, tempo);
      ++_tempoSN;
      }

//---------------------------------------------------------
//   delTempo
//---------------------------------------------------------

void TempoList::delTempo(int tick)
      {
      del(tick);
      ++_tempoSN;
      }

//---------------------------------------------------------
//   changeTempo
//---------------------------------------------------------

void TempoList::changeTempo(int tick, int newTempo)
      {
      change(tick, newTempo);
      ++_tempoSN;
      }

//---------------------------------------------------------
//   setMasterFlag
//---------------------------------------------------------

bool TempoList::setMasterFlag(int /*tick*/, bool val)
      {
      if (useList != val) {
            useList = val;
            ++_tempoSN;
            return true;
            }
      return false;
      }

//---------------------------------------------------------
//   tick2time
//---------------------------------------------------------

double TempoList::tick2time(int tick, double time, int* sn) const
      {
      return (*sn == _tempoSN) ? time : tick2time(tick, sn);
      }

//---------------------------------------------------------
//   time2tick
//    return cached value t if list did not change
//---------------------------------------------------------

int TempoList::time2tick(double time, int t, int* sn) const
      {
      return (*sn == _tempoSN) ? t : time2tick(time, sn);
      }

//---------------------------------------------------------
//   tick2time
//---------------------------------------------------------

double TempoList::tick2time(int tick, int* sn) const
      {
      double t;
      if (useList) {
            ciTEvent i = upper_bound(tick);
            int delta = tick - i->second->tick;
            t = i->second->time + double(delta) / (division * _globalTempo * 10000.0 / i->second->tempo);
            }
      else
            t = (double(tick) * double(_tempo)) / (double(division) * _globalTempo * 10000.0);
      if (sn)
            *sn = _tempoSN;
      return t;
      }

//---------------------------------------------------------
//   time2tick
//---------------------------------------------------------

int TempoList::time2tick(double time, int* sn) const
      {
      int tick;
      if (useList) {
            ciTEvent e;
            for (e = begin(); e != end();) {
                  ciTEvent ee = e;
                  ++ee;
                  if (ee == end())
                        break;
                  if (time < ee->second->time)
                        break;
                  e = ee;
                  }
            int te = e->second->tempo;
            double dtime = time - e->second->time;
            tick = e->second->tick + lrint(dtime * _globalTempo * division * 10000.0 / te);
            }
      else
            tick = lrint(time * _globalTempo * division * 10000.0 / double(_tempo));
      if (sn)
            *sn = _tempoSN;
      return tick;
      }

//---------------------------------------------------------
//   TempoList::write
//---------------------------------------------------------

void TempoList::write(int level, Xml& xml) const
      {
      xml.put(level++, "<tempolist fix=\"%d\">", _tempo);
      if (_globalTempo != 100)
            xml.intTag(level, "globalTempo", _globalTempo);
      for (ciTEvent i = begin(); i != end(); ++i)
            i->second->write(level, xml, i->first);
      xml.tag(level, "/tempolist");
      }

//---------------------------------------------------------
//   TempoList::read
//---------------------------------------------------------

void TempoList::read(Xml& xml)
      {
      for (;;) {
            Xml::Token token = xml.parse();
            const QString& tag = xml.s1();
            switch (token) {
                  case Xml::Error:
                  case Xml::End:
                        return;
                  case Xml::TagStart:
                        if (tag == "tempo") {
                              TEvent* t = new TEvent();
                              int tick = t->read(xml);
                              iTEvent pos = find(tick);
                              if (pos != end())
                                    erase(pos);
                              insert(std::pair<const int, TEvent*> (tick, t));
                              }
                        else if (tag == "globalTempo")
                              _globalTempo = xml.parseInt();
                        else
                              xml.unknown("TempoList");
                        break;
                  case Xml::Attribut:
                        if (tag == "fix")
                              _tempo = xml.s2().toInt();
                        break;
                  case Xml::TagEnd:
                        if (tag == "tempolist") {
                              normalize();
                              ++_tempoSN;
                              return;
                              }
                  default:
                        break;
                  }
            }
      }

//---------------------------------------------------------
//   TEvent::write
//---------------------------------------------------------

void TEvent::write(int level, Xml& xml, int at) const
      {
      xml.tag(level++, "tempo at=\"%d\"", at);
      xml.intTag(level, "tick", tick);
      xml.intTag(level, "val", tempo);
      xml.tag(level, "/tempo");
      }

//---------------------------------------------------------
//   TEvent::read
//---------------------------------------------------------

int TEvent::read(Xml& xml)
      {
      int at = 0;
      for (;;) {
            Xml::Token token = xml.parse();
            const QString& tag = xml.s1();
            switch (token) {
                  case Xml::Error:
                  case Xml::End:
                        return 0;
                  case Xml::TagStart:
                        if (tag == "tick")
                              tick = xml.parseInt();
                        else if (tag == "val")
                              tempo = xml.parseInt();
                        else
                              xml.unknown("TEvent");
                        break;
                  case Xml::Attribut:
                        if (tag == "at")
                              at = xml.s2().toInt();
                        break;
                  case Xml::TagEnd:
                        if (tag == "tempo") {
                              return at;
                              }
                  default:
                        break;
                  }
            }
      return 0;
      }
